/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.bonita.approvalWorkflow;

import java.io.InputStream;
import java.util.Collection;
import java.util.Map;

import javax.security.auth.login.LoginContext;
import javax.security.auth.login.LoginException;

import org.ow2.bonita.facade.ManagementAPI;
import org.ow2.bonita.facade.QueryDefinitionAPI;
import org.ow2.bonita.facade.QueryRuntimeAPI;
import org.ow2.bonita.facade.RuntimeAPI;
import org.ow2.bonita.facade.def.majorElement.ProcessDef;
import org.ow2.bonita.facade.runtime.ActivityInst;
import org.ow2.bonita.facade.runtime.TaskInst;
import org.ow2.bonita.facade.uuid.PackageDefinitionUUID;
import org.ow2.bonita.facade.uuid.ProcessDefinitionUUID;
import org.ow2.bonita.facade.uuid.ProcessInstanceUUID;
import org.ow2.bonita.facade.uuid.TaskUUID;
import org.ow2.bonita.util.Constants.TaskState;
import org.ow2.novabpm.util.Misc;
import org.ow2.novabpm.util.SimpleCallbackHandler;
import org.ow2.novabpm.util.StandardCallbackHandler;

/**
 * @author "Pierre Vigneras", "Marc Blachon", "Charles Souillard"
 */
public class ApprovalWorkflow {
  public static final String PROCESS_ID = "ApprovalWorkflow";
  private LoginContext loginContext;

  public void execute(RuntimeAPI runtimeAPI, QueryRuntimeAPI queryRuntimeAPI,
      ManagementAPI managementAPI, QueryDefinitionAPI queryDefinitionAPI, String barFile) throws Exception {
    ProcessDef process = deploy(managementAPI, barFile).get(PROCESS_ID);
    PackageDefinitionUUID packageUUID = process.getPackageDefinitionUUID();
    Misc.badStateIfNull(packageUUID, "packageUUID is null in process : " + process.getProcessId());
    ProcessInstanceUUID instanceUUID = launch(packageUUID, runtimeAPI, queryRuntimeAPI, queryDefinitionAPI, process.getUUID());
    runtimeAPI.deleteInstance(instanceUUID);
    managementAPI.undeploy(packageUUID);
  }

  public void login() throws LoginException {
    String loginMode = System.getProperty("login.mode");
    if (loginMode != null && "test".equals(loginMode)) {
      loginContext = new LoginContext("Bonita", new SimpleCallbackHandler("John", "bonita"));
    } else {
      loginContext = new LoginContext("Bonita", new StandardCallbackHandler());
    }
    loginContext.login();
  }
  public void logout() throws LoginException {
    loginContext.logout();
  }

  protected ProcessInstanceUUID launch(PackageDefinitionUUID packageUUID,
      RuntimeAPI runtimeAPI, QueryRuntimeAPI queryRuntimeAPI, QueryDefinitionAPI queryDefinitionAPI,
      ProcessDefinitionUUID processUUID) throws Exception {
    final ProcessInstanceUUID instanceUUID = runtimeAPI.instantiateProcess(processUUID);
    final Collection<ActivityInst< TaskInst >> activities = queryRuntimeAPI.getTaskList(instanceUUID, TaskState.READY);

    if (activities.isEmpty()) {
      throw new Exception("No task found? Bad User?");
    }
    for (ActivityInst< TaskInst > activity : activities) {
      TaskUUID taskUUID = activity.getBody().getUUID();
      runtimeAPI.startTask(taskUUID);
      System.out.println("Starting task: " + taskUUID);
      runtimeAPI.finishTask(taskUUID);
      System.out.println("Task " + taskUUID + " finished.");
    }
    return instanceUUID;
  }

  protected Map<String, ProcessDef> deploy(ManagementAPI managementAPI, String barFile) throws Exception {
    if (barFile == null) {
      System.err.println("Usage: " + ApprovalWorkflow.class + " bpar file");
      System.exit(1);
    }
    final InputStream in = ApprovalWorkflow.class.getResourceAsStream(barFile);
    if (in == null) {
      throw new Exception("Can't find ressource: " + barFile);
    }
    byte[] bar = Misc.getAllContentFrom(in);
    return managementAPI.deployBar(bar);
  }

}
